use crate::{spotify, youtube};
use chrono::{Date, Utc};
use std::time::Duration;

pub(crate) enum MusicData {
    Track(Track),
    Album(Album),
}

pub(crate) struct Track {
    name: String,
    authors: Vec<Author>,
    duration: Duration,
    album: Vec<Album>,
    description: String,
    lyrics: String,
}

pub(crate) struct Album {
    name: String,
    authors: Vec<Author>,
    description: String,
    year: Date<Utc>,
}

pub(crate) struct Author {
    name: String,
    surname: String,
    date_of_birth: Date<Utc>,
}

// The enum holds all the currently supported type of Id which the engine can search for
pub(crate) enum ServiceIdKind {
    Spotify(String),
    Youtube(String),
    Automatic(String),
}

// This struct will allow us in the future to search, cache and store data and metadata regarding
// tracks, albums and playlists
pub(crate) struct MusicEngine {
    spotify: spotify::Client,
    youtube: youtube::Client,
}

impl MusicEngine {
    pub(crate) async fn new() -> Self {
        MusicEngine {
            spotify: spotify::Client::new().await,
            youtube: youtube::Client::new().await,
        }
    }

    pub(crate) fn new_with_dependencies(
        spotify_client: spotify::Client,
        youtube_client: youtube::Client,
    ) -> Self {
        MusicEngine {
            spotify: spotify_client,
            youtube: youtube_client,
        }
    }

    pub(crate) async fn search_by_name(&self, name: &str) {
        todo!("In the future it would be possible to search for all metadata on a record from this call")
    }
    pub(crate) async fn search_by_id(&self, id: ServiceIdKind) {
        match id {
            ServiceIdKind::Spotify(id) => {
                let entry_kind = spotify::get_entry_kind(id.as_str());
            }
            ServiceIdKind::Youtube(id) => {}
            ServiceIdKind::Automatic(id) => {}
        }
    }
}
